#! /usr/bin/env python
################################################################################
# Description: These are some sample tests                                     #
# They are based on the WP API                                                 #
# Author: Jasmine-Arabella Post                                                      #
# Date Started: 20171122                                                       #
################################################################################


#########################
#    Imports & Setup    #
#########################
import pytest
import inspect
from random import choice
from string import ascii_letters
import logging
import datetime
from random_word import RandomWords
import json
from apis import WP_APIs
import os

# imports from local code
import repackage
repackage.up(1)
from pyrest.util import AssertTest

# setup time stamp
now = datetime.datetime.now()
time = now.strftime("%d--%H%M%S")
number_name = now.strftime("%H%M%S")



#    Setup Logging
#=================
from pyrest import setup
myLogger = logging.getLogger('myLogger')
myLogger.setLevel(logging.DEBUG)

# Set environment to test in
# NOTE: must be done after import setup
env = str(os.getenv("envURL"))


############################################################
#                  Tests Starts Here                       #
############################################################

def test_tmp():
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    test = WP_APIs.GetPages(env)
    test.CallAPI()
    test.Call_Succeeded(test)

    # Trying to figure out the issue with my assertTest function
    AssertTest(testObject=test, assertTest = str(len(test.content)) + " > 10",
                       message="There was no data sent back from the call")

    myLogger.info("This test passed\n")


def test_Get_Posts():
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    test = WP_APIs.GetPosts(env)
    test.CallAPI()
    assert (len(test.content) > 0)
    test.Call_Succeeded(test)
    myLogger.info("This test passed\n")


def test_Get_Posts_Call_Succeeded():
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    test = WP_APIs.GetPosts(env)
    test.CallAPI()
    assert (len(test.content) > 0)
    test.Call_Succeeded(test)
    myLogger.info("This test passed\n")


def test_Get_Posts_Call_Failed():
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    test = WP_APIs.GetPosts_bad(env)
    test.CallAPI()
    assert (len(test.content) > 0)
    test.Call_Failed(test, return_code="404", returns_content='yes')
    myLogger.info("This test passed\n")


def test_Make_Good_New_Post(capsys):
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    post_title = 'This is cow number ' + number_name
    content = ''.join(choice(ascii_letters) for i in range(120))
    data = {'status': 'publish', 'Content-Type': 'application/x-www-form-urlencoded'}
    data['title'] = post_title
    data['content_raw'] = content
    test = WP_APIs.PostNewPost(user='testAdmin01', data=data, env=env)
    test.CallAPI()

    if not len(test.content) > 0:
        assert len(test.content) > 0
        out, err = capsys.readouterr()
        myLogger.error("Out put: ", out)
        myLogger.error("Error is: " + err)
        myLogger.error("This test FAILED")
        return

    if not test.status == int("201"):
        assert test.status == int("201"), "We didn't get the expected return code"
        out, err = capsys.readouterr()
        myLogger.error("Out put: ")
        myLogger.error("Error is: ", err)
        myLogger.error("This test FAILED")
        return
    test.Call_Succeeded(test, return_code="201")
    myLogger.info("This test passed\n")


def test_Make_Failed_New_Post():
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    post_title = 'This is the ' + ''.join(choice(ascii_letters) for i in range(12)) + ' big dead cow'
    content = ''.join(choice(ascii_letters) for i in range(120))
    data = {'status': 'publish', 'Content-Type': 'application/x-www-form-urlencoded'}
    data['title'] = post_title
    data['content_raw'] = content
    test = WP_APIs.PostNewPost(user='', data=data, env=env)
    test.CallAPI()
    assert (test.status == int("401"))
    assert (str(test.content).find("Sorry, you are not allowed to create posts as this user.") > 0 )
    test.Call_Failed(test, return_code='401', returns_content='yes')
    myLogger.info("This test passed\n")


def test_Make_Good_New_Post_2(capsys):
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    post_title = 'There are ' + number_name + ' cows'
    rWords = RandomWords()
    content = ' '.join(rWords.random_word() for i in range(600))
    data = {'status': 'draft', 'Content-Type': 'application/x-www-form-urlencoded'}
    data['title'] = post_title
    data['content_raw'] = content
    test = WP_APIs.PostNewPost(user='testAdmin01', data=data, env=env)
    test.CallAPI()

    if not len(test.content) > 0:
        assert len(test.content) > 0
        out, err = capsys.readouterr()
        myLogger.error("Out put: ", out)
        myLogger.error("Error is: " + err)
        myLogger.error("This test FAILED")
        return

    if not test.status == int("201"):
        assert test.status == int("201"), "We didn't get the expected return code"
        out, err = capsys.readouterr()
        myLogger.error("Out put: ")
        myLogger.error("Error is: ", err)
        myLogger.error("This test FAILED")
        return
    test.Call_Succeeded(test, return_code="201")
    myLogger.info("This test passed\n")


# End of File
# ==============================================================================



#! /usr/bin/env python
########################################################################################################################
# Description: This file contains the base class for calling API's                                                     #
# Author: Jasmine-Arabella Post                                                                                        #
# Date Started: 20160922                                                                                              #
########################################################################################################################


#########################
#    Setup Stuff      #
#########################
import requests
import json
import re
import logging
from . util import *


# the following is to prevent warnings about insecure connection because
# we don't have proper certs on our interal servers
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

# setting up logging
callLogger = logging.getLogger('myLogger')
callLogger.setLevel(logging.NOTSET)


############################################################
#                     API Class                        #
############################################################
class API(object):
    """
    This is the base class for all APIs that are to be tested.  This class uses the requests library to make the call
    and has other methods for analyzing the results.
    """

    def __init__(self, address, data="", header="", method="GET", params="", user="", password=""):
        """
        :param address: --REQUIRED-- The full url being called.
        :param data: Any and all data that needs to be sent as part of the payload
        :param header: The API call headers
        :param method: What request method should be used
        :param params: Any URL parameters that need to be passed
        :param user: The log in user if needed
        :param password: The log in users password if needed
        """
        self.address = address
        self.params = params
        self.method = method
        self.header = header
        self.user = user
        self.password = password
        self.data = data

    ########################################
    #          Method to call API          #
    ########################################
    def CallAPI(self):
        """
        This method makes the actual API call.
        After a successful API call the following attributes are available.
          <object>.json is the JSON returned from the call
          <object>.content is the content returned, JSON, text or otherwise
          <object>.return_headers is the text of the returned headers
          <object>.url is the called URL
          <object>.status is the returend status
          <object>.body is the request body
        :return: the return status of the call made
        """
        global info, testFAILED
        testFAILED = "false"

        def logFailedTest():
            """
            This little method is just used to log failed API calls
            :return: Nothing
            """
            global testFAILED
            callLogger.error("= = = = = = = = = = = Error = = = = = = = = = = = ")
            callLogger.error("The framework failed to call the API ")
            callLogger.error("The method used is: " + self.method)
            callLogger.error("The url called is: " + self.address)
            callLogger.error("The header sent is: " + str(self.header))
            callLogger.error("The date sent is: " + str(self.data))
            callLogger.error("= = = = = = = = = = Test Failed = = = = = = = = = = \n")

            testFAILED = "true"
            # raise  AssertionError


        
        #   Make the API call
        #====================

        # Setup request parameters
        request_params = {"method": self.method, "url": self.address}
        if self.header != "":
            request_params["headers"] = self.header

        if self.params != "":
            request_params["params"] = self.params

        if self.data != "":
            request_params["data"] = self.data

        # Make call
        if self.user:
            try:
                info = requests.request(auth=(self.user, self.password), **request_params)
            except:
                logFailedTest()
        else:
            try:
                info = requests.request(**request_params)
            except:
                logFailedTest()

        # for debug what was sent
        callLogger.debug("                                ---New API Call---")
        callLogger.debug("The method used is: " + self.method)
        callLogger.debug("The url called is: " + self.address)
        callLogger.debug("The header sent is: " + str(self.header))
        callLogger.debug("The date sent is: " + str(self.data))

        # what was returned
        if not testFAILED == "true":
            callLogger.debug("The return code was: " + str(info.status_code))
            callLogger.debug("The header returned is: " + str(info.headers))
            callLogger.debug("The content returned is: " + str(info.content))

            # is the return code json or http
            if re.search(r'DOCTYPE html', str(info.content), flags=re.I):
                print("\n\nCall returned HTML, not json")
                callLogger.error("The API call returned HTML")
                return "FAILED"

        
        #  set some attributes
        #=====================
        # list of attributes
        listAttrib = ["self.json = info.json()", "self.content = info.content", "self.return_headers = info.headers",
                     "self.url = info.url", "self.status = info.status_code", "self.body = info.request.body",
                     "self.info = info"]
        # if URL was called loop through list of attributes to create
        if not testFAILED == "true":
            for atrib in listAttrib:
                try:
                    exec(atrib) in globals(), locals()
                except:
                    callLogger.error("Failed to create attribute: " + atrib)
                    pass
            returnMessage = self.status
        else:
            returnMessage = "Test Failed"

        return returnMessage

    ########################################
    #      Verify API call succeeded       #
    ########################################
    def Call_Succeeded(self, testObject, return_code='200', returns_content='yes'):
        """
        This method does a basic verification that the API call succeeded

        :param testObject: --REQUIRED-- the object created by the test
        :param return_code: the return code expected.    Default is 200
        :param returns_content: The return data much contain content.    Default yes
        """
      
        # some logging
        callLogger.info("The status is: " + str(self.status))

        # some basic asserts
        if returns_content == 'yes':
            AssertTest(testObject=testObject, assertTest="len(testObject.content) > 0",
                       message="There was no data sent back from the call", assertError='no')

        # specific assert
        assert_test = "testObject.status == " + return_code
        assert_message = "The test didn't return a " + return_code + " as expected"
        AssertTest(testObject=self, assertTest=assert_test, message=assert_message, assertError='no')
        callLogger.info("This call succeeded as expected.       TEST PASSED\n")

    ########################################
    #       Verify API call fails          #
    ########################################
    def Call_Failed(self, testObject, return_code='400', returns_content='no'):
        """
        This method does a basic verification that the API call failed
       
        :param testObject: --REQUIRED-- the object created by the test
        :param return_code: the return code expected.    Default is 400
        :param returns_content: The return data much contain content.    Default no
        """

        # some logging
        callLogger.info("The status is: " + str(self.status))

        # some basic asserts
        if returns_content == 'yes':
            AssertTest(testObject=testObject, assertTest="len(testObject.content) > 0",
                       message="There was no data sent back from the call", assertError='no')
        elif returns_content == 'no':
            AssertTest(testObject=testObject, assertTest="len(testObject.content) == 0",
                       message="There was unexpected data sent back from the call", assertError='no')

        # specific assert
        assert_test = "testObject.status == " + return_code
        assert_message = "The test didn't return a " + return_code + " as expected"
        AssertTest(testObject=self, assertTest=assert_test, message=assert_message, assertError='no')
        callLogger.info("This call -failed- as expected.        TEST PASSED\n")


    ########################################
    #   Method to pretty print the JSON    #
    ########################################
    def JSON_PP(self):
        """
        This is a simple method to pretty print the returned JSON
        It is mainly used in interactive testing.
        :return: Nothing, but the JSON is printed to StdOut
        """
        global info
        json1 = json.loads(info.content)
        print( json.dumps(json1, indent=4, sort_keys=True))
        
        
# End of File
# ==============================================================================


#! /usr/bin/env python
########################################################################################################################
# Description: atr is the "API Test Runner" to run API tests via py.tesrt                                                         #
# Author: Jasmine-Arabella Post                                                                                        #
# Date Started:  20161028                                                                                              #
########################################################################################################################


#########################
#      Setup Stuff      #
#########################
import os
import argparse
import logging
from argparse import RawTextHelpFormatter
from subprocess import call
import re



#   Process CLI Options
#======================
app_description = """ATR is API Test Runner
----------------------
This test runner will take a py.test formatted file
and run it in the given environment.
"""
parser = argparse.ArgumentParser(description=app_description, formatter_class=RawTextHelpFormatter)

# Required parameters
required = parser.add_argument_group("Required")
required.add_argument('-e', '--environment', dest='env', action="store", default="qa",
                      choices=['dev', 'qa', 'prod'],
                      help="This sets the server environment the test will be run against")
required.add_argument('-f', '--file2test', dest='test_file', action='store', required=True,  help="The file to test (jmx).")

# optional parameters
parser.add_argument('-d', '--debug', dest="debug", action="store_true",  default=False,
                    help="This option sets logging to debug mode. The default is INFO")
parser.add_argument('-v', '--variables', dest="vars", action="store",
                    help="This option allows you to use a CSV file to add more global variables to your test suite")

userOptions = parser.parse_args()


#   Set Environment to Test In
#=============================
# build path to file
envFile = userOptions.env + "-environment.csv"
cwd = os.getcwd()

# need to get the path out of test_file
# then replayc /pyrest/ with that
path4test = os.path.dirname(userOptions.test_file)
path2file = "{}/{}/env/{}".format(cwd, path4test, envFile)

# Verify path to environment configuration file
if not os.path.isfile(path2file):
    print( "Could not find '" + str(path2file) + "', the environment file.  There for tests can not be run.")
    exit()

# Set path file to OS env var
os.environ["pyTestEnv"] = path2file



#    Set Log Level
#=================
# Defaults to INFO, but can be set to DEBUG
if userOptions.debug:
    os.environ["pyTestDebug"] = 'true'


#   Set Other Variables
#======================
if userOptions.vars:
    os.environ["pyTestVars"] = userOptions.vars


########################################
#           Call py.test               #
########################################

# check for specific test
look4test = re.compile('::.*')
bearFile = look4test.sub("", userOptions.test_file)


# Verify path to test file
if not os.path.isfile(bearFile):
    print( "Could not find '" + bearFile + "', the test file.  There for tests can not be run.")
    exit()

os.environ["pyTestFile"] = userOptions.test_file

# Call py.test
call(["py.test","-v", userOptions.test_file])


# End of File
# ==============================================================================


#! /usr/bin/env python
################################################################################
# Description: These are some sample tests                                     #
# They are based on the WP API                                                 #
# Author: Jasmine-Arabella Post                                                      #
# Date Started: 20171122                                                       #
################################################################################


#########################
#    Imports & Setup    #
#########################
import pytest
import inspect
import logging
import datetime
from apis import WP_APIs
import os

# imoports from local files
import repackage
repackage.up(1)
from pyrest.util import *

# Set time stamp
now = datetime.datetime.now()
time = now.strftime("%d--%H%M%S")
number_name = now.strftime("%H%M%S")


#    Setup Logging
#=================
from pyrest import setup
myLogger = logging.getLogger('myLogger')
myLogger.setLevel(logging.DEBUG)

# Set environment to test in
# NOTE: must be done after import setup
env = str(os.getenv("envURL"))


############################################################
#                  Tests Starts Here                       #
############################################################

def test_tmp():
    test_name = inspect.stack()[0][3]
    myLogger.info("Test " + test_name + " is starting")
    test = WP_APIs.GetPages(env=env)
    test.CallAPI()
    test.Call_Succeeded(test)

    # Trying to figure out the issue with my assertTest function
    AssertTest(testObject=test, assertTest = str(len(test.content)) + " > 10",
                       message="There was no data sent back from the call")

    myLogger.info("This test passed\n")


# End of File
# ==============================================================================
